<?php

namespace Drupal\liveness\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class LivenessSettingsForm extends ConfigFormBase {

  protected function getEditableConfigNames() {
    return ['liveness.settings'];
  }

  public function getFormId() {
    return 'liveness_settings_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('liveness.settings');
    $environments = $config->get('environments') ?? [];

    // Always set environments in form state
    if (!$form_state->has('environments')) {
      $form_state->set('environments', $environments);
    } else {
      $environments = $form_state->get('environments');
    }

    $form['#tree'] = TRUE;
    $form['#prefix'] = '<p><a href="/admin/config/development/performance/liveness/events">View Liveness Log</a></p>';

    $form['enable_liveness_probing'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Liveness Probing'),
      '#default_value' => $config->get('enable_liveness_probing') ?? TRUE,
    ];

    $form['enable_liveness_block'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Liveness Notification Block'),
      '#default_value' => $config->get('enable_liveness_block') ?? TRUE,
    ];

    $form['display_events_timeframe'] = [
      '#type' => 'number',
      '#title' => $this->t('Display events timeframe (hours)'),
      '#description' => $this->t('Enter the number of hours to display events in the liveness block.'),
      '#default_value' => $config->get('display_events_timeframe') ?? 48,
      '#min' => 1,
    ];

    $form['environments'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Environments'),
      '#prefix' => '<div id="environments-wrapper">',
      '#suffix' => '</div>',
    ];

    $num_environments = $form_state->get('num_environments');
    if ($num_environments === NULL) {
      $num_environments = count($environments) ?: 1;
      $form_state->set('num_environments', $num_environments);
    }

    for ($i = 0; $i < $num_environments; $i++) {
      $form['environments'][$i] = [
        '#type' => 'details',
        '#title' => $this->t('Environment @number', ['@number' => $i + 1]),
        '#open' => TRUE,
      ];
      $form['environments'][$i]['name'] = [
        '#type' => 'textfield',
        '#title' => $this->t('Name'),
        '#default_value' => isset($environments[$i]['name']) ? $environments[$i]['name'] : '',
        '#required' => TRUE,
      ];
      $form['environments'][$i]['url'] = [
        '#type' => 'url',
        '#title' => $this->t('URL'),
        '#default_value' => isset($environments[$i]['url']) ? $environments[$i]['url'] : '',
        '#required' => TRUE,
      ];
      $form['environments'][$i]['enabled'] = [
        '#type' => 'checkbox',
        '#title' => $this->t('Enable probing'),
        '#default_value' => isset($environments[$i]['enabled']) ? $environments[$i]['enabled'] : TRUE,
      ];
    }

    $form['add_environment'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another environment'),
      '#submit' => ['::addEnvironment'],
      '#ajax' => [
        'callback' => '::addEnvironmentCallback',
        'wrapper' => 'environments-wrapper',
      ],
    ];
    $form['remove_environment'] = [
      '#type' => 'submit',
      '#value' => $this->t('Remove environment'),
      '#submit' => ['::removeEnvironment'],
      '#ajax' => [
        'callback' => '::removeEnvironmentCallback',
        'wrapper' => 'environments-wrapper',
      ],
    ];

    $form['notification_emails'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Notification Emails'),
      '#description' => $this->t('Enter email addresses separated by commas.'),
      '#default_value' => $config->get('notification_emails'),
    ];

    return parent::buildForm($form, $form_state);
  }

  public function addEnvironment(array &$form, FormStateInterface $form_state) {
    $num_environments = $form_state->get('num_environments');
    $form_state->set('num_environments', $num_environments + 1);
    $form_state->setRebuild(TRUE);
  }

  public function addEnvironmentCallback(array &$form, FormStateInterface $form_state) {
    return $form['environments'];
  }

  public function removeEnvironment(array &$form, FormStateInterface $form_state) {
    $num_environments = $form_state->get('num_environments');
    if ($num_environments) {
      $form_state->set('num_environments', $num_environments - 1);
      $form_state->setRebuild(TRUE);
    }
  }

  public function removeEnvironmentCallback(array &$form, FormStateInterface $form_state) {
    return $form['environments'];
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $submitted_values = $form_state->getValue('environments');
    $environments = [];
    if (is_array($submitted_values)) {
      foreach ($submitted_values as $key => $value) {
        if (!empty($value['name']) && !empty($value['url'])) {
          $environments[] = $value;
        }
      }
    }

    $this->config('liveness.settings')
      ->set('environments', $environments)
      ->set('notification_emails', $form_state->getValue('notification_emails'))
      ->set('enable_liveness_probing', $form_state->getValue('enable_liveness_probing'))
      ->set('enable_liveness_block', $form_state->getValue('enable_liveness_block'))
      ->set('display_events_timeframe', $form_state->getValue('display_events_timeframe'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValue('environments');
    foreach ($values as $key => $fieldset_values) {
      if (isset($fieldset_values['name'])) {
        if (strlen($fieldset_values['name']) < 2) {
          $form_state->setErrorByName("environments][$key][name", $this->t('Please provide a valid name.'));
        }
      } else {
        $form_state->setErrorByName('name', $this->t('Please provide a valid name.'));
      }
      if (isset($fieldset_values['url'])) {
        if (strlen($fieldset_values['url']) < 8 || stripos($fieldset_values['url'], 'http') !== 0) {
          $form_state->setErrorByName("environments][$key][url", $this->t('Please provide a valid external URL.'));
        }
      }
    }
  }
}

